<?php
/*
Plugin Name: PB MailCrypt - AntiSpam E-Mail Encryption
Plugin URI: http://wordpress.org/extend/plugins/pb-mailcrypt-antispam-email-encryption/
Description: This Plugin provides you an easy to use Shortcode to encrypt email links and protect them against spam bots.
Version: 2.0.0
Author: Pascal Bajorat
Author URI: http://www.pascal-bajorat.com
Text Domain: pbMailCrypt
Domain Path: /lang
License: GNU General Public License v.3

Copyright (c) 2016 by Pascal Bajorat.
*/

class pbMailCrypt {
	const linkClass = 'mailcrypt';
	const replaceChar = '∂';
	const emailPattern = '/[a-z0-9!#$%&\'*+\/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&\'*+\/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/';
	const emailPattern2 = '/>[a-z0-9!#$%&\'*+\/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&\'*+\/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])<?/';
	const emailPatternWithLink = '/<a[^>]+>[a-z0-9!#$%&\'*+\/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&\'*+\/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])<\/a>?/';

	const setting_pb_mailcrypt_auto_mode = 'pb_mailcrypt_auto_mode';
	const setting_pb_mailcrypt_link = 'pb_mailcrypt_link';

	/**
	 * pbMailCrypt constructor.
	 *
	 * @since 1.0.0
	 */
	public function init()
	{
		/*
		 * Load language files
		 */
		load_plugin_textdomain('pbMailCrypt', false, dirname(plugin_basename( __FILE__ )).'/lang/');

		/*
		 * Add JavaScript files
		 */
		wp_register_script('pbMailCrypt', plugins_url('mailcrypt.js', __FILE__), array('jquery'), '1.0.0', true);
		add_action('wp_enqueue_scripts', array(__CLASS__, 'enqueue_script'));

		/*
		 * Shortcodes
		 */
		add_shortcode('mailcrypt', array(__CLASS__, 'mailcrypt'));
		add_shortcode('mail', array(__CLASS__, 'mailcrypt'));

		/*
		 * Filter
		 */
		add_filter('the_content', array(__CLASS__, 'encrypt_mails_in_content'), 20);
		add_filter('plugin_action_links_'.plugin_basename(__FILE__), array(__CLASS__, 'settings_link'));

		/*
		 * Settings
		 */
		add_action('admin_init' , array(__CLASS__, 'register_fields'));
	}

	/**
	 * wp_enqueue_scripts action
	 *
	 * @since 1.0.0
	 */
	public function enqueue_script()
	{
		wp_enqueue_script('jquery');
		wp_enqueue_script('pbMailCrypt');
	}

	/**
	 * Mailcrypt Shortcode
	 *
	 * @param array $atts
	 * @since 1.0.0
	 * @return string
	 */
	public static function mailcrypt( $atts )
	{
		extract( shortcode_atts( array(
			'email' => false,
			'href' => false,
			'title' => false
		), $atts ) );

		/** @var string|boolean $href */
		$hrefOption = get_option(self::setting_pb_mailcrypt_link);
		if( !empty($hrefOption) && $href == false ) {
			$href = $hrefOption;
		} elseif( $href == false ) {
			$href = '#';
		}

		if( is_numeric($href) ) {
			$href = get_permalink($href);

			if( empty($href) ) {
				$href = home_url();
			}
		}

		/** @var string $email */
		$emailSplit = @explode('@', $email);
		
		if( !empty($email) && is_array($emailSplit) ) {
			/** @var string|boolean $title */
			$email = '<a href="' . $href . '" ' . (( $title != false)? 'title="' . $title . '"':'') . ' class="' . self::linkClass . '">' . $emailSplit[0] . '<span><span>' . self::replaceChar . '</span></span>' . $emailSplit[1] . '</a>';
		}
		
		return $email;
	}

	/**
	 * Function to encrypt email addresses in the_content
	 *
	 * @param $email
	 *
	 * @return string
	 */
	public static function mailcrypt_content( $email )
	{
		$match = preg_match(self::emailPattern2, $email[0], $matches);

		if( $match ) {
			$email[0] = trim($matches[0], '><');
		}

		return self::mailcrypt(array(
			'email' => $email[0]
		));
	}

	/**
	 * Content Filter
	 *
	 * @param $content
	 *
	 * @return mixed
	 */
	public static function encrypt_mails_in_content( $content )
	{
		if( get_option(self::setting_pb_mailcrypt_auto_mode) == true ) {
			$content = preg_replace_callback(self::emailPatternWithLink, array(__CLASS__, 'mailcrypt_content'), $content);
			$content = preg_replace_callback(self::emailPattern, array(__CLASS__, 'mailcrypt_content'), $content);
		}

		return $content;
	}

	/**
	 * Add settings link to Plugins overview
	 *
	 * @param $data
	 *
	 * @return array
	 */
	public static function settings_link( $data )
	{
		if( ! current_user_can('manage_options') ) {
			return $data;
		}

		return array_merge(
			$data,
			array(
				sprintf(
					'<a href="%s#pb_mailcrypt_options_section">%s</a>',
					add_query_arg(
						array(),
						admin_url('options-general.php')
					),
					__('Settings', 'pbMailCrypt')
				)
			)
		);
	}

	/**
	 * Section heading
	 */
	function section()
	{
		echo '<p><strong>'.__('Here you can enable the automatic email encryption mode for PB MailCrypt. If you have problems with the automatic email encryption or  special cases that not be auto detected you can use the following shortcode:', 'pbMailCrypt').'</strong></p>';

		echo '<p><code>[mailcrypt email="name@example.com" href="'.__('Alternative Contact-Page URL', 'pbMailCrypt').'" title="'.__('A-Tag title attribute', 'pbMailCrypt').'"]</code></p>';

		echo '<p>'.sprintf(
				__('PB MailCrypt is a free WordPress Plugin by <a href="%s" target="_blank">Pascal Bajorat</a> and made with %s in Berlin, Germany. If you like it and maybe want to <a href="%s" target="_blank">buy me a cup of coffee or a beer</a> I would appreciate that very much.', 'pbMailCrypt'),
				'http://www.pascal-bajorat.com',
				'<span style="color: #f00;">&#9829;</span>',
				'http://www.pascal-bajorat.com/spenden/'
			).'</p>';
	}

	/**
	 * Setting fields
	 */
	public static function register_fields()
	{
		add_settings_section(
			'pb_mailcrypt_options_section',
			__('PB MailCrypt - AntiSpam E-Mail Encryption', 'pbMailCrypt').'<a name="pb_mailcrypt_options_section" id="pb_mailcrypt_options_section"></a>',
			array(__CLASS__, 'section'),
			'general'
		);

		register_setting( 'general', 'pb_mailcrypt_auto_mode', 'esc_attr' );
		add_settings_field(
			'pb_mailcrypt_auto_mode',
			'<label for="extra_blog_desc_id">'.__('Auto mode', 'pbMailCrypt').'</label>',
			array(__CLASS__, 'fields_html'),
			'general',
			'pb_mailcrypt_options_section',
			array(
				'id' => 'pb_mailcrypt_auto_mode',
				'section' => 'pb_mailcrypt_options',
				'type' => 'checkbox',
				'default' => '',
				'desc' => __('enable automatic email encryption', 'pbMailCrypt')
			)
		);

		register_setting( 'general', 'pb_mailcrypt_link', 'esc_attr' );
		add_settings_field(
			'pb_mailcrypt_link',
			'<label for="extra_blog_desc_id">'.__('Link or Page-ID', 'pbMailCrypt').'</label>',
			array(__CLASS__, 'fields_html'),
			'general',
			'pb_mailcrypt_options_section',
			array(
				'id' => 'pb_mailcrypt_link',
				'section' => 'pb_mailcrypt_options',
				'type' => 'text',
				'default' => '/',
				'desc' => __('Alternative Contact-Page URL', 'pbMailCrypt')
			)
		);
	}

	/**
	 * Render settings
	 *
	 * @param $args
	 */
	public static function fields_html( $args )
	{
		$option = get_option($args['id']);
		$html = '';

		if( $args['type'] == 'text' ) {
			if ( empty( $option ) ) {
				$val = $args['default'];
			} else {
				$val = $option;
			}
			$html = '<input type="text" id="' . $args['id'] . '" name="' . $args['id'] . '" class="regular-text" value="' . $val . '" />';
			if ( ! empty( $args['desc'] ) ) {
				$html .= '<p class="description">' . $args['desc'] . '</p>';
			}
		} elseif( $args['type'] == 'checkbox' ) {
			if( $option === false ){
				$val = $args['default'];
			}else{
				$val = $option;
			}
			$html = '<input type="checkbox" id="'.$args['id'].'" name="'.$args['id'].'" value="1" '.checked(1, $val, false).'/>';
			$html .= '<label for="'.$args['id'].'"> '.$args['desc'].'</label>';
		}

		echo $html;
	}

	/**
	 * Uninstall PB MailCrypt
	 */
	public static function uninstall()
	{
		/* Global */
		/** @var object $wpdb */
		global $wpdb;

		/* Remove settings */
		delete_option('pb_mailcrypt_auto_mode');
		delete_option('pb_mailcrypt_link');

		/* Clean DB */
		$wpdb->query("OPTIMIZE TABLE `" .$wpdb->options. "`");
	}
}

add_action(
	'plugins_loaded',
	array(
		'pbMailCrypt',
		'init'
	)
);

register_uninstall_hook(
	__FILE__,
	array(
		'pbMailCrypt',
		'uninstall'
	)
);